#include "bleas.h"

#define AUDIO_SVC_UUID 		             0xb1, 0x48, 0xcb, 0x35, 0x05, 0x4d, 0xc2, 0x87, 0xcf, 0x48, 0x9d, 0x71, 0x98, 0x13, 0x7a, 0x19
#define AUDIO_CHAR_UUID					 0xe8, 0x06, 0xbe, 0x70, 0x15, 0x3b, 0xdf, 0xa0, 0xb5, 0x4f, 0xf3, 0xb3, 0x66, 0xc2, 0xe0, 0x10
#define SENSOR_CHAR_UUID				 0xea, 0x06, 0xbe, 0x70, 0x15, 0x3b, 0xdf, 0xa0, 0xb5, 0x4f, 0xf3, 0xb3, 0x66, 0xc2, 0xe0, 0x10

static struct bt_uuid_128 audio_svc_uuid 		 = BT_UUID_INIT_128(AUDIO_SVC_UUID);
static struct bt_uuid_128 audio_uuid       		 = BT_UUID_INIT_128(AUDIO_CHAR_UUID);
static struct bt_uuid_128 sensor_uuid       	 = BT_UUID_INIT_128(SENSOR_CHAR_UUID);
 
static ssize_t audio_read_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, void *buf, uint16_t len, uint16_t offset);
static ssize_t audio_write_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, const void *buf, uint16_t len, uint16_t offset, uint8_t flags);
static void audio_ccc_changed(const struct bt_gatt_attr *attr, uint16_t value);

static ssize_t sensor_read_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, void *buf, uint16_t len, uint16_t offset);
static ssize_t sensor_write_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, const void *buf, uint16_t len, uint16_t offset, uint8_t flags);
static void sensor_ccc_changed(const struct bt_gatt_attr *attr, uint16_t value);

struct k_poll_signal signal_audio_action;
struct k_poll_signal signal_sensor_action;
struct k_poll_signal signal_udp_action;

volatile BLE_INFO g_BLE_INFO;

/* BLE Service Declaration */
BT_GATT_SERVICE_DEFINE(audio_svc,
	BT_GATT_PRIMARY_SERVICE(&audio_svc_uuid),
	
	BT_GATT_CHARACTERISTIC(
                           &audio_uuid.uuid, 
                           BT_GATT_CHRC_READ | BT_GATT_CHRC_WRITE | BT_GATT_CHRC_NOTIFY, 
                           BT_GATT_PERM_READ | BT_GATT_PERM_WRITE, 
                           audio_read_raw_data, 
                           audio_write_raw_data, 
                           NULL),
                       BT_GATT_CCC(audio_ccc_changed, BT_GATT_PERM_READ | BT_GATT_PERM_WRITE),
                       BT_GATT_CUD("AUDIO", BT_GATT_PERM_READ),	  
                       
    BT_GATT_CHARACTERISTIC(
                           &sensor_uuid.uuid, 
                           BT_GATT_CHRC_READ | BT_GATT_CHRC_WRITE | BT_GATT_CHRC_NOTIFY, 
                           BT_GATT_PERM_READ | BT_GATT_PERM_WRITE, 
                           sensor_read_raw_data, 
                           sensor_write_raw_data, 
                           NULL),
                       BT_GATT_CCC(sensor_ccc_changed, BT_GATT_PERM_READ | BT_GATT_PERM_WRITE),
                       BT_GATT_CUD("SENSOR", BT_GATT_PERM_READ),	  
		    
	/* more optional Characteristics */
);

/**
 *  @brief  This callback is executed when the client is attempting to read back the from AUDIO char
 *  @param  struct bt_conn* - The connection that is requesting to read
 *  @param  const struct bt_gatt_attr* - The attribute that’s being read
 *  @param  void* - Buffer where to place the status and, if available, the read data
 *  @param  uint16_t - Length of data to read
 *  @param  uint16_t - Offset to start reading from
 *  @return ssize_t - Number fo bytes read, or in case of an error BT_GATT_ERR() with a specific ATT error code
 */
ssize_t audio_read_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, void *buf, uint16_t len, uint16_t offset) 
{
    //printk("-> BLEAS: audio data read len=%d, offset=%d\n", len, offset);

	return 0;
}

/**
 *  @brief  This callback is executed when the write is executed on AUDIO char
 *  @param  struct bt_conn* - The connection that is requesting to write
 *  @param  const struct bt_gatt_attr* - The attribute that’s being written
 *  @param  const void* - Buffer containing the request's information
 *  @param  uint16_t - Number of bytes in the buffer
 *  @param  uint16_t - Offset to start writing from
 *  @param  uint8_t - Flags (BT_GATT_WRITE_*)
 *  @return ssize_t - Number of bytes written, or in case of an error BT_GATT_ERR() with a specific ATT error code
 */
ssize_t audio_write_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, const void *buf, uint16_t len, uint16_t offset, uint8_t flags) 
{
    uint32_t k, n = 0;
    uint32_t size = 256;
    //printk("-> BLEAS: len=%d, offset=%d\n\t", len, offset);
    uint8_t * ptr = (uint8_t *)buf;
    
    // buffer input data
	for(k=0;k<len;k++)
	{
		g_BLE_INFO.rx_buff[g_BLE_INFO.rx_buff_widx] = ptr[k];
		//printk("%02X ", ptr[k]);
		if(k == 2)
		{
			size = 4 + (uint32_t)ptr[k];
		}
		
		g_BLE_INFO.rx_buff_widx++;
		if(g_BLE_INFO.rx_buff_widx >= (BLE_BUFFER_SIZE))
		{
			g_BLE_INFO.rx_buff_widx = 0;
		}
		
		n++;
		if(n >= size)
		{
			break;
		}
	}
	//printk("\n");
    k_poll_signal_raise(&signal_audio_action, 0);
    return 0;
}

/**
 *  @brief  This callback is executed when the client changed the permission on audio characteristic.
 *  @param  const struct bt_gatt_attr* - The attribute that’s being modified
 *  @param  uint16_t - Client characteristic configuration access permissions
 *  @return None
 */
void audio_ccc_changed(const struct bt_gatt_attr *attr, uint16_t value) 
{
	if(value == (BT_GATT_CCC_NOTIFY))
	{
		g_BLE_INFO.notification_state = (BLE_NOTIFICATION_ENABLED);
	}
	else
	{
		g_BLE_INFO.notification_state = (BLE_NOTIFICATION_DISABLED);
	}
	
	printk("-> BLEAS: AUDIO CCCD[%d]\n", g_BLE_INFO.notification_state);
    
}

void txDataASBLE(struct bt_conn *active_conn, uint8_t * data, uint16_t size)
{
	if (g_BLE_INFO.notification_state == (BLE_NOTIFICATION_ENABLED)) 
	{
		notify(active_conn, &audio_uuid.uuid, &audio_svc.attrs[0], data, size);
	}
}

void initASBLE(void)
{
	//g_BLE_INFO.state = (BLE_DEV_STATE_IDLE);
	g_BLE_INFO.notification_state = (BLE_NOTIFICATION_DISABLED);
	g_BLE_INFO.sensor_notification_state = (BLE_NOTIFICATION_DISABLED);
	g_BLE_INFO.cflag = (BLE_NOT_CONNECTED);
	g_BLE_INFO.rx_buff_widx = 0;
	g_BLE_INFO.rx_buff_ridx = 0;
}

/**
 *  @brief  This callback is executed when the client is attempting to read back the from SENSOR char
 *  @param  struct bt_conn* - The connection that is requesting to read
 *  @param  const struct bt_gatt_attr* - The attribute that’s being read
 *  @param  void* - Buffer where to place the status and, if available, the read data
 *  @param  uint16_t - Length of data to read
 *  @param  uint16_t - Offset to start reading from
 *  @return ssize_t - Number fo bytes read, or in case of an error BT_GATT_ERR() with a specific ATT error code
 */
ssize_t sensor_read_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, void *buf, uint16_t len, uint16_t offset) 
{
    //printk("-> BLEAS: sensor read data len=%d, offset=%d\n", len, offset);
	return 0;
}

/**
 *  @brief  This callback is executed when the write is executed on SENSOR char
 *  @param  struct bt_conn* - The connection that is requesting to write
 *  @param  const struct bt_gatt_attr* - The attribute that’s being written
 *  @param  const void* - Buffer containing the request's information
 *  @param  uint16_t - Number of bytes in the buffer
 *  @param  uint16_t - Offset to start writing from
 *  @param  uint8_t - Flags (BT_GATT_WRITE_*)
 *  @return ssize_t - Number of bytes written, or in case of an error BT_GATT_ERR() with a specific ATT error code
 */
ssize_t sensor_write_raw_data(struct bt_conn *conn, const struct bt_gatt_attr *attr, const void *buf, uint16_t len, uint16_t offset, uint8_t flags) 
{
	//uint32_t k, n = 0;
    //printk("-> BLEAS: sensor write data len=%d, offset=%d\n\t", len, offset);
    //uint8_t * ptr = (uint8_t *)buf;
    
    // display received data from the write operation initiated via Mobile App
	//for(k=0;k<len;k++)
	//{
		////printk("%02X ", ptr[k]);
	//}
	//printk("\n");
	
    //k_poll_signal_raise(&signal_sensor_action, 0);
    return 0;
}

/**
 *  @brief  This callback is executed when the client changed the permission on SENSOR characteristic.
 *  @param  const struct bt_gatt_attr* - The attribute that’s being modified
 *  @param  uint16_t - Client characteristic configuration access permissions
 *  @return None
 */
void sensor_ccc_changed(const struct bt_gatt_attr *attr, uint16_t value) 
{
	if(value == (BT_GATT_CCC_NOTIFY))
	{
		g_BLE_INFO.sensor_notification_state = (BLE_NOTIFICATION_ENABLED);
	}
	else
	{
		g_BLE_INFO.sensor_notification_state = (BLE_NOTIFICATION_DISABLED);
	}
	
	printk("-> BLEAS: SENSOR CCCD[%d]\n", g_BLE_INFO.notification_state);
}

void txSensorDataASBLE(struct bt_conn *active_conn, uint8_t * data, uint16_t size)
{
	if (g_BLE_INFO.sensor_notification_state == (BLE_NOTIFICATION_ENABLED)) 
	{
		notify(active_conn, &sensor_uuid.uuid, &audio_svc.attrs[0], data, size);
	}
}

/**
 *  @brief  Executed when an attribute value changed. Notify only when notification is enabled.
 *  @param  const struct bt_uuid* - The UUID. If the server contains multiple services with the same
 *                                  UUID, then the first occurrence, starting from the attr given, is used.
 *  @param  const void* - Buffer with the data to write
 *  @param  uint16_t - Number of bytes in the buffer
 *  @return int - Number of bytes written, or in case of an error BT_GATT_ERR() with a specific ATT error code
 */
int notify(struct bt_conn *active_conn, const struct bt_uuid *uuid, const struct bt_gatt_attr *attr, const void *data, uint16_t len) {
    int err;

    if (active_conn == NULL) {   
        return 0;
    } 

    uint16_t mtu = bt_gatt_get_mtu(active_conn);

    // Notification need an header of 3 bytes (struct bt_att_notify defined in zephyr/subsys/bluetooth/host/att_internal.h)
    len = MIN(len, mtu - 3);
printf("send the data\n");
    err = bt_gatt_notify_uuid(active_conn, uuid, attr, data, len);
    if (err) {
        printk("Unable to send notification (err %d)\n", err);
    }

    return err;
}



