#include "cmdapi.h"



volatile CMDAPI_INFO g_CMDAPI_INFO[CMDAPI_SIZE] = 
{
    {
        .cmd = CMD_FW_CAPTURE_STATUS_REQ,
        .exeCmd = getCaptureStatusCMDAPI,
    },
    {
        .cmd = CMD_FW_SET_CAPTURE_STATE,
        .exeCmd = setCaptureStateCMDAPI,
    },
    {
        .cmd = CMD_FW_STREAM_INIT_REQ,
        .exeCmd = streamInitReqCMDAPI,
    },
    {
        .cmd = CMD_FW_STREAM_DATA_REQ,
        .exeCmd = getDataCMDAPI,
    },
    {
        .cmd = CMD_FW_UDP_REQ,
        .exeCmd = udpInitReqCMDAPI,
    },
};

void getCaptureStatusCMDAPI(void)
{
	uint8_t data[9] = {0};
	uint32_t widx = g_AUDIO_INFO.widx;
	uint32_t cap_size = g_AUDIO_INFO.cap_size;
	
	if(g_AUDIO_INFO.state == (AUDIO_STATE_IDLE))
	{
		data[0] = (AUDIO_STATUS_IDLE);
	}
	else if(g_AUDIO_INFO.state == (AUDIO_STATE_WAIT4CAPTURE_END))
	{
		data[0] = (AUDIO_STATUS_CAPTURE);
		data[1] = (uint8_t)((widx>>24)&0x000000FF);
		data[2] = (uint8_t)((widx>>16)&0x000000FF);
		data[3] = (uint8_t)((widx>>8)&0x000000FF);
		data[4] = (uint8_t)((widx)&0x000000FF);
		data[5] = (uint8_t)((cap_size>>24)&0x000000FF);
		data[6] = (uint8_t)((cap_size>>16)&0x000000FF);
		data[7] = (uint8_t)((cap_size>>8)&0x000000FF);
		data[8] = (uint8_t)((cap_size)&0x000000FF);
	}
	else if(g_AUDIO_INFO.state == (AUDIO_STATE_WAIT4CAPTURE_END))
	{
		data[0] = (AUDIO_STATUS_DATA_READY);
		data[1] = (uint8_t)((widx>>24)&0x000000FF);
		data[2] = (uint8_t)((widx>>16)&0x000000FF);
		data[3] = (uint8_t)((widx>>8)&0x000000FF);
		data[4] = (uint8_t)((widx)&0x000000FF);
		data[5] = (uint8_t)((cap_size>>24)&0x000000FF);
		data[6] = (uint8_t)((cap_size>>16)&0x000000FF);
		data[7] = (uint8_t)((cap_size>>8)&0x000000FF);
		data[8] = (uint8_t)((cap_size)&0x000000FF);
	}
	
	
	printk("-> CMDAPI: tx-ing status [%02X] [%d - %d]\n",data[0], widx, cap_size);
	txCMD(CMD_FW_CAPTURE_STATUS_REP, 0, data, 9);
}

void setCaptureStateCMDAPI(void)
{
	printk("-> CMDAPI: action mode %d\n",g_CMD_INFO.payload[0][0]);
	if(g_CMD_INFO.payload[0][0] == 1)
	{
		g_AUDIO_INFO.state = (AUDIO_STATE_INIT);
		
		// initiate the capture and set the capture limit size in number of samples
		g_AUDIO_INFO.cap_size = ((uint32_t)g_CMD_INFO.payload[0][1])<<24;
		g_AUDIO_INFO.cap_size |= ((uint32_t)g_CMD_INFO.payload[0][2])<<16;
		g_AUDIO_INFO.cap_size |= ((uint32_t)g_CMD_INFO.payload[0][3])<<8;
		g_AUDIO_INFO.cap_size |= ((uint32_t)g_CMD_INFO.payload[0][4]);
	}
	else
	{
		g_AUDIO_INFO.state = (AUDIO_STATE_CAPTURE_COMPLETED);
		g_AUDIO_INFO.cap_size = g_AUDIO_INFO.widx;
	}
}

void streamInitReqCMDAPI(void)
{
	uint8_t data[4] = {0};
	uint32_t cap_size;
	if(g_AUDIO_INFO.state == (AUDIO_STATE_WAIT4CAPTURE_END))
	{// if streaming initiated while capture in progess stop the capture
		g_AUDIO_INFO.state = (AUDIO_STATE_CAPTURE_COMPLETED);
		g_AUDIO_INFO.cap_size = g_AUDIO_INFO.widx;
	}
	cap_size = g_AUDIO_INFO.cap_size;
	
	// reply only holds 4 bytes defining number of captured samples
	data[0] = (uint8_t)((cap_size>>24)&0x000000FF);
	data[1] = (uint8_t)((cap_size>>16)&0x000000FF);
	data[2] = (uint8_t)((cap_size>>8)&0x000000FF);
	data[3] = (uint8_t)((cap_size)&0x000000FF);
	
	txCMD(CMD_FW_STREAM_INIT_REP, 0, data, 4);
}

void getDataCMDAPI(void)
{
	uint8_t data[BLE_PAYLOAD_SIZE] = {0};
	uint32_t addr;
	uint32_t size = 0;
	
	
	
	// request will hold the relative position of the first byte in the payload 4B
	g_AUDIO_INFO.ridx = ((uint32_t)g_CMD_INFO.payload[0][0])<<24;
	g_AUDIO_INFO.ridx |= ((uint32_t)g_CMD_INFO.payload[0][1])<<16;
	g_AUDIO_INFO.ridx |= ((uint32_t)g_CMD_INFO.payload[0][2])<<8;
	g_AUDIO_INFO.ridx |= ((uint32_t)g_CMD_INFO.payload[0][3]);
	
	if(g_AUDIO_INFO.ridx > (AUDIO_MAX_CAPTURE_SIZE))
	{
		g_AUDIO_INFO.ridx = (AUDIO_MAX_CAPTURE_SIZE);
	}
	
	
	data[0] = (uint8_t)((g_AUDIO_INFO.ridx>>24)&0x000000FF);
	data[1] = (uint8_t)((g_AUDIO_INFO.ridx>>16)&0x000000FF);
	data[2] = (uint8_t)((g_AUDIO_INFO.ridx>>8)&0x000000FF);
	data[3] = (uint8_t)((g_AUDIO_INFO.ridx)&0x000000FF);
	size = 4;
	
	addr = g_AUDIO_INFO.ridx*2;
	
	size = (BLE_PAYLOAD_SIZE) - 8;
	if((addr +  (BLE_PAYLOAD_SIZE) - 8) >= (ERAM_MEMORY_SIZE))
	{
		size = (ERAM_MEMORY_SIZE) - addr;
	}
	
	// check the address if it will go beyond die addr boundary
	if((addr < (ERAM_DIE_ADDR_BOUNDARY))&&((addr+size) > (ERAM_DIE_ADDR_BOUNDARY)))
	{
		uint32_t tsize = (ERAM_DIE_ADDR_BOUNDARY) - addr;
		// write segment in first die
		readDataERAM(addr, &data[4], tsize);
		
		// write remaining in second die
		readDataERAM(ERAM_DIE_ADDR_BOUNDARY, &data[4+tsize], size - tsize);
		
		printk("-> ERAM: read boundary case:\n\tADD: %08X\n\t1RD: %d\n\t2RD: %d\n",addr,tsize, size-tsize);
	}
	else
	{	
		readDataERAM(addr, &data[4], size);
	}
	
	printk("-> CMDAPI: tx-ing [%08X - %08X] SIZE[%d]\n",g_AUDIO_INFO.ridx,g_AUDIO_INFO.cap_size,size);
	
	// reply will hold relative address and payload size - 4 bytes of data
	txCMD(CMD_FW_STREAM_DATA_REP, 0, data, size+4);
}

void udpInitReqCMDAPI(void)
{
	uint8_t data[4] = {0};
	uint32_t cap_size;
	if(g_AUDIO_INFO.state == (AUDIO_STATE_WAIT4CAPTURE_END))
	{// if streaming initiated while capture in progess stop the capture
		g_AUDIO_INFO.state = (AUDIO_STATE_CAPTURE_COMPLETED);
		g_AUDIO_INFO.cap_size = g_AUDIO_INFO.widx;
	}
	
	g_BLE_INFO.udp_timestamp = getSYSTIM();
	g_BLE_INFO.udp_state = 1;
	g_BLE_INFO.udp_stream++;
	g_BLE_INFO.udp_pkt_cnt = 0;
	g_BLE_INFO.udp_data_idx = 0;
	k_poll_signal_raise(&signal_udp_action, 0);
	printk("-> CMDAPI: UDP transfer initiated SIZE: %d STREAM: %d  TIME: %d ms\n", g_AUDIO_INFO.cap_size,g_BLE_INFO.udp_stream,getElapsedTimeSYSTIM(g_BLE_INFO.udp_timestamp) );
}

void txUdpDataCMDAPI(void)
{
	
	uint8_t data[BLE_PAYLOAD_SIZE] = {0};
	uint32_t addr;
	uint32_t size = 0;
	uint16_t k = 0;
	
	uint32_t timestamp = getElapsedTimeSYSTIM(g_BLE_INFO.udp_timestamp);
	
	// METADATA is 11 B remaining is 233B of BLE payload
	// stream ID
	data[k++] = g_BLE_INFO.udp_stream;
	// stream SIZE
	uint32_t cap_size = g_AUDIO_INFO.cap_size*2;
	data[k++] = (uint8_t)((cap_size>>24)&0x000000FF);
	data[k++] = (uint8_t)((cap_size>>16)&0x000000FF);
	data[k++] = (uint8_t)((cap_size>>8)&0x000000FF);
	data[k++] = (uint8_t)((cap_size)&0x000000FF);
	// stream PACKET NUMBER
	data[k++] = (uint8_t)((g_BLE_INFO.udp_pkt_cnt>>8)&0x000000FF);
	data[k++] = (uint8_t)((g_BLE_INFO.udp_pkt_cnt)&0x000000FF);
	// stream timestamp
	data[k++] = (uint8_t)((timestamp>>24)&0x000000FF);
	data[k++] = (uint8_t)((timestamp>>16)&0x000000FF);
	data[k++] = (uint8_t)((timestamp>>8)&0x000000FF);
	data[k++] = (uint8_t)((timestamp)&0x000000FF);
	
	
	addr = g_BLE_INFO.udp_data_idx;
	
	size = (BLE_PAYLOAD_SIZE) - 4-12;
	if((addr +  (BLE_PAYLOAD_SIZE) - 4-12) >= (ERAM_MEMORY_SIZE))
	{
		size = (ERAM_MEMORY_SIZE) - addr;
	}
	
	// check the address if it will go beyond die addr boundary
	if((addr < (ERAM_DIE_ADDR_BOUNDARY))&&((addr+size) > (ERAM_DIE_ADDR_BOUNDARY)))
	{
		uint32_t tsize = (ERAM_DIE_ADDR_BOUNDARY) - addr;
		// read segment in first die
		readDataERAM(addr, &data[k], tsize);
		
		// read remaining in second die
		readDataERAM(ERAM_DIE_ADDR_BOUNDARY, &data[k+tsize], size - tsize);
		
		printk("-> ERAM: read boundary case:\n\tADD: %08X\n\t1RD: %d\n\t2RD: %d\n",addr,tsize, size-tsize);
	}
	else
	{	
		readDataERAM(addr, &data[k], size);
	}
	
	//for(k=0;k<228;k++)
	//{
		//data[11+k] = k;
	//}
	
	
	g_BLE_INFO.udp_pkt_cnt++;
	printk("-> CMDAPI: tx-ing [%08X - %08X] SIZE[%d] %d\n",g_BLE_INFO.udp_data_idx, g_AUDIO_INFO.cap_size*2,size,g_BLE_INFO.udp_pkt_cnt);
	// reply will hold relative address and payload size - 4 bytes of data
	txCMD(CMD_FW_UDP_REP, 0, data, size+11);
	
	g_BLE_INFO.udp_data_idx += size;
	if(g_BLE_INFO.udp_data_idx < (g_AUDIO_INFO.cap_size*2))
	{
		k_poll_signal_raise(&signal_udp_action, 0);
	}
	else
	{
		printk("-> CMDAPI: transfer completed in %d ms of size %d B\n",timestamp, g_AUDIO_INFO.cap_size);
	}
	//k_msleep(100);
}




